package org.wegra.awt;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * @author kharaz + Elassad
 */
public class SimpleGrid extends JPanel implements Grid, Scrollable {
	private static final Border BLUE_BORDER
		= BorderFactory.createLineBorder(Color.blue);
	private static final Border BLACK_BORDER
		= BorderFactory.createLineBorder(Color.black);
	
	/** fixed layout manager of SimpleGrid */
	private SimpleGridLayout fLayout = new SimpleGridLayout();
	/** internal elements */
	private JLabel[][] fCells;
	
	/**
	 * Creates a instance.
	 */
	public SimpleGrid() {
		this(null);
	}
	
	/**
	 * Creates new instance with elements.
	 * 
	 * @param elements Elements of grid.
	 */
	public SimpleGrid(String[][] elements) {
		if ( elements != null ) {
			setElements(elements);
		}
		setBackground(Color.white);
		setLayout(fLayout);
	}
	
	
	public void setElements(Object[][] elements) {
		setElements(null, null, (String[][])elements);
	}
	

	public void setElements(String[] rowNames, String[] columnNames,
			String[][] elements)
	{
		removeAll();
		int row = elements.length + 1;
		int column = elements[0].length + 1;

		fCells = new JLabel[row][column];
		
		// 0, 0
		fCells[0][0] = new JLabel("");
		fCells[0][0].setBorder(BLUE_BORDER);
		fCells[1][0] = new JLabel("");
		fCells[1][0].setBorder(BLUE_BORDER);
		fCells[0][1] = new JLabel("");
		fCells[0][1].setBorder(BLUE_BORDER);
		
		// row header
		if ( rowNames == null ) {
			for ( int r = 2; r < row; r++ ) {
				JLabel label = new JLabel("");
				label.setBorder(BLUE_BORDER);
				fCells[r][0] = label;
			}
		} else {
			for ( int r = 2; r < row; r++ ) {
				JLabel label = new JLabel( rowNames[r - 2] );
				label.setBorder(BLUE_BORDER);
				label.setHorizontalAlignment(SwingConstants.CENTER);
				fCells[r][0] = label;
			}
		}
		
		// column header
		if ( columnNames == null ) {
			for ( int c = 2; c < column; c++ ) {
				JLabel label = new JLabel("");
				label.setBorder(BLUE_BORDER);
				fCells[0][c] = label;
			}
		} else {
			for ( int c = 2; c < column; c++ ) {
				JLabel label = new JLabel( columnNames[c - 2] );
				label.setBorder(BLUE_BORDER);
				label.setHorizontalAlignment(SwingConstants.CENTER);
				fCells[0][c] = label;
			}
		}
		
		for ( int r = 1; r < row; r++ ) {
			for ( int c = 1; c < column; c++ ) {
				JLabel label = new JLabel(elements[r-1][c-1]);
				label.setHorizontalAlignment(SwingConstants.RIGHT);
				label.setBorder(BLACK_BORDER);
				fCells[r][c] = label;
			}
		}
		
		for ( int r = 0; r < row; r++ ) {
			for ( int c = 0; c < column; c++ ) {
				add(fCells[r][c]);
			}
		}
		
		fLayout.layoutContainer(this);
	}

	public final int getRowCount() {
		return ( fCells != null ) ? fCells.length : 0;
	}

	public final int getColumnCount() {
		return ( fCells != null ) ? fCells[0].length : 0;
	}
	
	public void setCellSize(int width, int height) {
		((SimpleGridLayout)getLayout()).setCellSize(width, height);
	}

	public void setCellColor(int row, int column, Color color) {
		if ( row < fCells.length && column < fCells[0].length
			&& row >= 0 && column >= 0 )
		{
			fCells[row][column].setForeground(color);
		}
	}
	
	
	public final void setLayout(LayoutManager layout) {
		
	}
	

	public void removeAll() {
		super.removeAll();
		repaint();
	}
	

    public Dimension getPreferredScrollableViewportSize() {
        return getPreferredSize();
    }
	
	public int getScrollableBlockIncrement(Rectangle visibleRect,
			int orientation, int direction)
	{
		switch(orientation) {
			case SwingConstants.VERTICAL:
				return visibleRect.height;
			case SwingConstants.HORIZONTAL:
				return visibleRect.width;
			default:
				throw new IllegalArgumentException("Invalid orientation: "
													+ orientation);
		}
	}  


    public int getScrollableUnitIncrement(Rectangle visibleRect,
    		int orientation, int direction)
    {
        switch(orientation) {
	        case SwingConstants.VERTICAL:
	            return visibleRect.height / 10;
	        case SwingConstants.HORIZONTAL:
	            return visibleRect.width / 10;
	        default:
	            throw new IllegalArgumentException("Invalid orientation: "
	            									+ orientation);
        }
    }

	
	public boolean getScrollableTracksViewportWidth() {
		if (getParent() instanceof JViewport) {
			return (((JViewport)getParent()).getWidth()
					> getPreferredSize().width);
		}
		return false;
	}


	public boolean getScrollableTracksViewportHeight() {
		if (getParent() instanceof JViewport) {
			return (((JViewport)getParent()).getHeight()
					> getPreferredSize().height);
		}
		return false;
	}
}

