package seq.awt;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import seq.engine.*;

/**
 * Main user interface frame.
 */
public class AlignFrame extends JFrame {
	private static final Font DEFAULT_FONT
		= new Font("Courier New", Font.PLAIN, 12);
		
	// componenten.
	private JTextField fMatchTextField = new JTextField("5");
	private JTextField fMismatchTextField = new JTextField("-4");
	private JTextField fGapTextField = new JTextField("-7");
	private JTextArea fSequenceATextArea = new JTextArea();
	private JTextArea fSequenceBTextArea = new JTextArea();
	private JTextArea fResultTextArea = new JTextArea();
	private JButton fAlignButton = new JButton("Align");
	private JButton fShowMatrixButton = new JButton("erzeugte Matrix");
	private JRadioButton fGlobalAlignRadioButton
			= new JRadioButton("Needleman-Wunsch");
	private JRadioButton fLocalAlignRadioButton
			= new JRadioButton("Smith-Waterman");
	
	private JTextField fValidCharsTextField = new JTextField();
	private JComboBox fValidCharsComboBox = new JComboBox(new String[] {
 "allalstring"
	});
	
	
	private Engine fEngine = new Engine();
	/**  matrix frame */
	private WorkingMatrixFrame fMatrixFrame = new WorkingMatrixFrame();
	
	
	public AlignFrame() {
		super("Sequence Alignment mit Needleman-Wunsch und Smith-Waterman(kharaz + Elassad) ");
		
		initComponents();
		
		fValidCharsTextField.setText(fEngine.getValidChars());
		
		addWindowListener( new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				exit();
			}
		});
		
		setSize(600, 450);
		setResizable(false);
	}
	
	/**
	 * Initializes components.
	 */
	private void initComponents() {
		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(fGlobalAlignRadioButton);
		buttonGroup.add(fLocalAlignRadioButton);
		fGlobalAlignRadioButton.setSelected(true);
		
		JLabel matchLabel = new JLabel("Match");
		JLabel mismatchLabel = new JLabel("Mismatch");
		JLabel gapLabel = new JLabel("Gap");
		JLabel sequenceALabel = new JLabel("Sequence A");
		JLabel sequenceBLabel = new JLabel("Sequence B");
		JLabel resultLabel = new JLabel("Result");
		
		fGlobalAlignRadioButton.setBounds(10, 10, 100, 20);
		fLocalAlignRadioButton.setBounds(120, 10, 100, 20);
		
		matchLabel.setBounds(325, 10, 40, 20);
		fMatchTextField.setBounds(370, 10, 30, 20);
		fMatchTextField.setHorizontalAlignment(SwingConstants.RIGHT);
		mismatchLabel.setBounds(415, 10, 60, 20);
		fMismatchTextField.setBounds(480, 10, 30, 20);
		fMismatchTextField.setHorizontalAlignment(SwingConstants.RIGHT);
		gapLabel.setBounds(525, 10, 25, 20);
		fGapTextField.setBounds(550, 10, 30, 20);
		fGapTextField.setHorizontalAlignment(SwingConstants.RIGHT);
		
		fValidCharsTextField.setBounds(115, 35, 190, 20);
		fValidCharsTextField.setHorizontalAlignment(SwingConstants.RIGHT);
		
		
		fSequenceATextArea.setFont(DEFAULT_FONT);
		fSequenceBTextArea.setFont(DEFAULT_FONT);
		fResultTextArea.setFont(DEFAULT_FONT);
		
		JScrollPane sequenceAPane = new JScrollPane(fSequenceATextArea);
		JScrollPane sequenceBPane = new JScrollPane(fSequenceBTextArea);
		JScrollPane resultPane = new JScrollPane(fResultTextArea);
		
		sequenceALabel.setBounds(10, 60, 100, 20);
		sequenceAPane.setBounds(10, 80, 570, 80);
		sequenceBLabel.setBounds(10, 160, 100, 20);
		sequenceBPane.setBounds(10, 180, 570, 80);
		
		fAlignButton.setBounds(280, 270, 120, 20);
		fShowMatrixButton.setBounds(410, 270, 170, 20);
		
		resultLabel.setBounds(10, 290, 100, 20);
		resultPane.setBounds(10, 310, 570, 100);
		
		Container contentPane = getContentPane();
		contentPane.setLayout(null);
		contentPane.add(fGlobalAlignRadioButton);
		contentPane.add(fLocalAlignRadioButton);
		contentPane.add(matchLabel);
		contentPane.add(fMatchTextField);
		contentPane.add(mismatchLabel);
		contentPane.add(fMismatchTextField);
		contentPane.add(gapLabel);
		contentPane.add(fGapTextField);

		contentPane.add(sequenceALabel);
		contentPane.add(sequenceAPane);
		contentPane.add(sequenceBLabel);
		contentPane.add(sequenceBPane);
		contentPane.add(fAlignButton);
		contentPane.add(resultLabel);
		contentPane.add(resultPane);
		contentPane.add(fShowMatrixButton);
		
		KeyHandler keyHandler = new KeyHandler();
		fSequenceATextArea.addKeyListener(keyHandler);
		fSequenceBTextArea.addKeyListener(keyHandler);
		
		fAlignButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				String validChars = fValidCharsTextField.getText();
				if ( validChars.length() == 0 ) {
					JOptionPane.showMessageDialog(AlignFrame.this,
						"Please check [Valid Characters] field.");
				} else {
					align();
				}
			}
		});
		
		fShowMatrixButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				showWorkingMatrix();
			}
		});
		
		fValidCharsComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				int index = ((JComboBox)ae.getSource()).getSelectedIndex();
				switch ( index ) {
					
					case 0:
						fValidCharsTextField.setText(
								new String(Engine.allalstring));
						break;
					default:
						fValidCharsTextField.setText("");
						break;
				}
			}
		});
		
		fShowMatrixButton.setEnabled(false);
	}
	
	/**
	 * Aligns current input sequences.
	 */
	public void align() {
		try {
			fResultTextArea.setText("");

			String seq1 = fSequenceATextArea.getText();
			String seq2 = fSequenceBTextArea.getText();
			int match = Integer.parseInt(fMatchTextField.getText());
			int mismatch = Integer.parseInt(fMismatchTextField.getText());
			int gap = Integer.parseInt(fGapTextField.getText());
			boolean global = fGlobalAlignRadioButton.isSelected();
			
			
			fEngine.setValidChars(fValidCharsTextField.getText());
			
			fEngine.process(seq1, seq2, match, mismatch, gap, global);
			
			// Processes result.
			StringBuffer result = new StringBuffer();
			result.append( fEngine.getResultSequenceA() );
			result.append( '\n' );
			result.append( fEngine.getMatchLine() );
			result.append( '\n' );
			result.append( fEngine.getResultSequenceB() );
			result.append( '\n' );
			result.append( '\n' );
			result.append( "match: " + fEngine.getMatchCount() );
			result.append( "   mismatch: " + fEngine.getMismatchCount() );
			result.append( "   gap: " + fEngine.getGapCount() );
			result.append( '\n' );
			if ( global ) {
				result.append( "total score: " + fEngine.getScore() );
			} else {
				result.append( "max score: " + fEngine.getScore() );
			}
			
			fResultTextArea.setText(result.toString());

			fMatrixFrame.setData( 
				fEngine.getValidInputSequenceA(),
				fEngine.getValidInputSequenceB(),
				fEngine.getMatrix(),
				fEngine.getTraceResult() );
			fShowMatrixButton.setEnabled(true);
		} catch ( NumberFormatException nfe ) {
			fShowMatrixButton.setEnabled(false);
			JOptionPane.showMessageDialog(	this,
											"Invalid data format.",
											"Warning",
											JOptionPane.OK_OPTION);
		} catch ( IllegalArgumentException iae ) {
			fShowMatrixButton.setEnabled(false);
			JOptionPane.showMessageDialog(	this,
											"Invalid sequence data.",
											"Warning",
											JOptionPane.OK_OPTION);
		}
	}
	
	
	private void showWorkingMatrix() {
		fMatrixFrame.setVisible(true);
	}
	
	
	private void exit() {
		System.exit(1);
	}
	

	private class KeyHandler extends KeyAdapter {
		public void keyPressed(KeyEvent ke) {
			if ( ke.getKeyChar() == '\t' ) {
				JTextArea source = (JTextArea)ke.getSource();
				source.setText(source.getText().trim());
				source.transferFocus();
			}
		}
	}
}
