package seq.engine;

/**
 * @author kharaz + Elassad
 */

final class Element {
	
	public static final int NORTH_WEST = 0;
	
	public static final int NORTH = 1;
	
	public static final int WEST = 2;
	
	
	private Element[] fFrom = new Element[3];
	
	
	private int fRow;
	
	private int fColumn;

	private int fValue;

	
	private boolean fMatched;
	
	/**
	 * Creates new instance.
	 * 
	 * @param row Row index on matrix.
	 * @param column Column index on matrix.
	 * @param value Value of element.
	 */
	public Element(int row, int column, int value) {
		fRow = row;
		fColumn = column;
		fValue = value;
	}
	
	/**
	 * Creates new instance with default value (0).
	 * 
	 * @param row Row index on matrix.
	 * @param column Column index on matrix.
	 */
	public Element(int row, int column) {
		this(row, column, 0);
	}
	
	/**
	 * Sets matched to current element.
	 * 
	 * @param match Matched or not.
	 */
	public void setMatched(boolean match) {
		fMatched = match;
	}
	
	/**
	 * Tests if current element is matched or not.
	 * 
	 * @return True if it's matched, otherwise false.
	 */
	public boolean isMatched() {
		return fMatched;
	}
	
	/**
	 * Returns previous elements as array.
	 * 
	 * @return Previous elements.
	 */
	public Element getFrom(int from) {
		return fFrom[from];
	}
	
	/**
	 * Tests if current element went from <code>from</code>.
	 * 
	 * @param from Possible direction.
	 * @return True or false.
	 */
	public boolean isFrom(int from) {
		return ( fFrom[from] != null ) ? true : false;
	}
	
	/**
	 * Sets the previous element and it's direction.
	 * 
	 * @param from Direction of previous element.
	 * @param elemnet Previous element.
	 */
	public void setFrom(int from, Element element) {
		fFrom[from] = element;
	}
	
	/**
	 * Returns row of current element.
	 * 
	 * @return Row of current elenent.
	 */
	public int getRow() {
		return fRow;
	}
	
	/**
	 * Returns column of current element.
	 * 
	 * @return Column of current element.
	 */
	public int getColumn() {
		return fColumn;
	}
	
	/**
	 * Return value of current element.
	 * 
	 * @return Value of current element.
	 */
	public int getValue() {
		return fValue;
	}
	
	/**
	 * Sets value of current element.
	 * 
	 * @param value New value.
	 */
	public void setValue(int value){
		fValue = value;
	}
}
